@@ -301,7 +301,7 @@ class Textnet(TextnetBase, FormalContext):
 
     Parameters
     ----------
-    data: DataFrame
+    data: TidyText
         * DataFrame of tokens with per-document counts, as created by
           `Corpus.tokenized` `Corpus.ngrams`, and `Corpus.noun_phrases`.
         * An incidence matrix relating documents to terms.
@@ -321,13 +321,15 @@ class Textnet(TextnetBase, FormalContext):
 
     Attributes
     ----------
+    graph : ig.Graph
+        Direct access to the underlying igraph object.
     im : `IncidenceMatrix`
         Incidence matrix of the bipartite graph.
     """
 
     def __init__(
         self,
-        data: pd.DataFrame,
+        data: TidyText,
         min_docs: int = 2,
         connected: bool = False,
         doc_attrs: Optional[dict[str, dict[str, str]]] = None,
@@ -337,21 +339,36 @@ class Textnet(TextnetBase, FormalContext):
         if data.empty:
             raise ValueError("Data is empty.")
         if isinstance(data, IncidenceMatrix):
-            self.im = data
+            self._matrix = data
         elif isinstance(data, (TidyText, pd.DataFrame)):
-            self.im = _im_from_tidy_text(data, min_docs)
+            self._matrix = _im_from_tidy_text(data, min_docs)
 
     @cached_property
     def graph(self):
         """Direct access to the underlying igraph object."""
-        g = _graph_from_im(self.im)
+        g = _graph_from_im(self._matrix)
         if self._doc_attrs:
             for name, attr in self._doc_attrs.items():
                 g.vs[name] = [attr.get(doc) for doc in g.vs["id"]]
         if self._connected:
             return giant_component(g)
-        else:
-            return g
+        return g
+
+    @cached_property
+    def im(self) -> IncidenceMatrix:
+        """Incidence matrix of the bipartite graph."""
+        if not self._connected:
+            return self._matrix
+        a = np.array(self.graph.get_incidence(types=self.node_types)[0]).astype(
+            "float64"
+        )
+        a[a == 1] = self.edges["weight"]
+        doc_count, _ = a.shape
+        im = IncidenceMatrix(
+            a, index=self.nodes["id"][:doc_count], columns=self.nodes["id"][doc_count:]
+        )
+        im.T.index.name = "term"
+        return im
 
     def project(
         self, *, node_type: Literal["doc", "term"], connected: Optional[bool] = False
